#ifndef FILEMANAGERSERVICE_H
#define FILEMANAGERSERVICE_H

#include <QDateTime>
#include <QDebug>
#include <QDir>
#include <QDomDocument>
#include <QGraphicsWebView>
#include <QFileInfo>
#include <QMap>
#include <QMessageBox>
#include <QNetworkReply>
#include <QNetworkRequest>
#include <QObject>
#include <QProcess>
#include <QUuid>
#include <QWebFrame>
#include "html5applicationviewer.h"

class Binder : public QObject
{
    Q_OBJECT
public:
    Binder(QWebFrame *webFrame);
    QString uuid();
    QString reply();
    void success(QString &ret);
    void error(QString &err);

private:
    QUuid m_uuid;
    QWebFrame *m_webFrame;
    QString m_reply;
};


class Metadata
{
public:
    Metadata(QFileInfo info);
    void appendChild(Metadata &metadata);
    QDomDocument document ();
    QDomElement element(QDomDocument &doc);
    QString toString();

private:
    QDomElement createDomElementFromFileInfo (QDomDocument &doc, const QFileInfo &info);

    QFileInfo m_info;
    QList<Metadata> m_children;
};


class FileManagerService : public QObject
{
    Q_OBJECT
public:
    explicit FileManagerService(QObject *parent, Html5ApplicationViewer *html5ApplicationViewer);
    
signals:
    void signal_network_finished(QNetworkAccessManager *manager);

public slots:
    QString bindService();
    void unbindService(QString uuid);
    void execute(QString uuid, QString method, QStringList args);
    QString reply(QString uuid);
    void debug(QString msg);
    void openurl(QString url);


private:
    void metadata(Binder *binder, QStringList &args);
    void parents(Binder *binder, QStringList &args);
    void children(Binder *binder, QStringList &args);
    void get_file(Binder *binder, QStringList &args);
    void put_file(Binder *binder, QStringList &args);
    void post_file(Binder *binder, QStringList &args);
    void mkdir(Binder *binder, QStringList &args);
    void remove(Binder *binder, QStringList &args);
    void save_as_image(Binder *binder, QStringList &args);
    void network(Binder *binder, QStringList &args);

    QMap<QUuid, Binder*> m_binders;
    Html5ApplicationViewer *m_html5ApplicationViewer;
    QGraphicsWebView *m_webView;

private slots:
    void addToJavaScript();
    void network_reply(QNetworkReply *reply);
    void network_finished(QNetworkAccessManager *manager);

};

#endif // FILEMANAGERSERVICE_H
